==> ocg_login-0.0.10.tar.gz: lib/src/repository/login/firebase.dart <==
import 'dart:io';

import 'package:firebase_auth/firebase_auth.dart'
    show
        FacebookAuthCredential,
        FacebookAuthProvider,
        FirebaseAuth,
        GoogleAuthProvider,
        OAuthProvider,
        PhoneAuthProvider;
import 'package:flutter/foundation.dart';
import 'package:flutter_facebook_login/flutter_facebook_login.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_login/src/models/user.dart';
import 'package:ocg_login/src/repository/login/base.dart';
import 'package:ocg_login/src/repository/user.dart';
import 'package:sign_in_with_apple/sign_in_with_apple.dart';

class LoginFirebaseRepository extends LoginRepository {
  final FirebaseAuth firebaseAuth;
  final TokenRepository tokenRepository;
  final LoginUserRepository userRepository;

  LoginFirebaseRepository(this.tokenRepository, {this.userRepository})
      : this.firebaseAuth = FirebaseAuth.instance;

  @override
  Future<AppResponse<AppToken>> createUserWithEmailAndPassword(
      String email, String password) async {
    try {
      var userCredential = await firebaseAuth.createUserWithEmailAndPassword(
          email: email, password: password);
      var token = AppToken.named(userId: userCredential.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials:
              LoginUser.emailCredentials(email: email, password: password),
          type: LoginType.Email);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<AppToken>> signInWithEmailAndPassword(
      String email, String password) async {
    try {
      var userCredential = await firebaseAuth.signInWithEmailAndPassword(
          email: email, password: password);
      var token = AppToken.named(userId: userCredential.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials:
              LoginUser.emailCredentials(email: email, password: password),
          type: LoginType.Email);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> isLoggedIn() async {
    var user = firebaseAuth.currentUser;
    return AppResponse.named(data: user != null);
  }

  @override
  Future<AppResponse<AppToken>> signInWithFacebook() async {
    try {
      FacebookLogin _facebookLogin = FacebookLogin();
      await _facebookLogin.logOut();
      var authentication = await _facebookLogin.logIn([]);
      switch (authentication.status) {
        case FacebookLoginStatus.cancelledByUser:
          return AppResponse.named(error: "Cancelled by user");
          break;
        case FacebookLoginStatus.loggedIn:
          FacebookAuthCredential credential = FacebookAuthProvider.credential(
            authentication.accessToken.token,
          );
          var facebookResponse = await firebaseAuth.signInWithCredential(
            credential,
          );

          var token = AppToken.named(userId: facebookResponse.user.uid);
          await tokenRepository.saveToken(token);
          LoginUser user = LoginUser.named(
              uid: token.userId,
              credentials: LoginUser.facebookCredentials(
                facebookId: authentication.accessToken.userId,
                // TODO
              ),
              type: LoginType.Facebook);
          if (userRepository != null) {
            await userRepository.signUp(user);
          }
          return AppResponse.named(data: token);
        case FacebookLoginStatus.error:
          return AppResponse.named(error: authentication.errorMessage);
          break;
        default:
      }
      return AppResponse.named(error: "Facebook login error");
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "Facebook login error");
    }
  }

  @override
  Future<AppResponse<AppToken>> signWithGoogle() async {
    try {
      GoogleSignIn _googleSign = GoogleSignIn();
      await _googleSign.signOut();
      var response = await _googleSign.signIn();
      var authentication = await response.authentication;
      var credential = GoogleAuthProvider.credential(
          accessToken: authentication.accessToken,
          idToken: authentication.idToken);
      var googleResponse = await firebaseAuth.signInWithCredential(credential);
      var token = AppToken.named(userId: googleResponse.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials: LoginUser.googleCredentials(
              googleId: response.id,
              email: response.email,
              name: response.displayName,
              photoUrl: response.photoUrl),
          type: LoginType.Google);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "Google Login Error");
    }
  }

  String _verificationId;

  @override
  Future<AppResponse<String>> sendOTP(
      String countryCode, String mobileNumber) async {
    try {
      var error;
      String realMobile = "+" + countryCode + mobileNumber;
      _verificationId = null;
      await firebaseAuth.verifyPhoneNumber(
          phoneNumber: realMobile,
          verificationCompleted: (credential) {},
          verificationFailed: (e) {
            error = e;
            debugPrint("Error $e");
          },
          codeAutoRetrievalTimeout: (val) {},
          codeSent: (verificationId, val) {
            _verificationId = verificationId;
          },
          timeout: Duration(seconds: 6));

      if (error != null) {
        return AppResponse.named(error: 'Error $error');
      }
      for (var i = 0; i < 10; i++) {
        await Future.delayed(Duration(seconds: 1));
        if (_verificationId != null) {
          return AppResponse.named(data: realMobile);
        }
      }
      return AppResponse.named(error: "Timeout Exception");
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<AppToken>> verifyOTP(
      String mobileNumber, String otp) async {
    try {
      var credential = await firebaseAuth.signInWithCredential(
          PhoneAuthProvider.credential(
              verificationId: _verificationId, smsCode: otp));
      var token = AppToken.named(userId: credential.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials: LoginUser.mobileCredentials(mobile: mobileNumber),
          type: LoginType.Mobile);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<AppToken>> signInWithApple() async {
    try {
      if (!Platform.isIOS) {
        return AppResponse.named(error: "Only supported for ios");
      }
      var result = await SignInWithApple.getAppleIDCredential(scopes: [
        AppleIDAuthorizationScopes.email,
        AppleIDAuthorizationScopes.fullName,
      ]);
      var appleCredential = result.authorizationCode;
      final oAuthProvider = OAuthProvider("apple.com");
      final credential = oAuthProvider.credential(
          accessToken: appleCredential, idToken: result.identityToken);
      var appleResponse = await firebaseAuth.signInWithCredential(credential);
      var token = AppToken.named(userId: appleResponse.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials: LoginUser.appleCredential(
            appleId: result.userIdentifier,
            email: result.email,
            name: result.givenName,
          ),
          type: LoginType.Google);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> signOut() async {
    try {
      GoogleSignIn().signOut();
      FacebookLogin().logOut();
      await firebaseAuth.signOut();
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<AppToken>> getToken() async {
    return await tokenRepository.getToken();
  }

  @override
  Future<AppResponse<bool>> sendResetPasswordEmail(String email) async {
    try {
      await firebaseAuth.sendPasswordResetEmail(email: email);
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }
}

==> ocg_login-0.0.14.tar.gz: lib/src/repository/login/firebase.dart <==
import 'dart:io';

import 'package:firebase_auth/firebase_auth.dart'
    show
        AuthCredential,
        FacebookAuthCredential,
        FacebookAuthProvider,
        FirebaseAuth,
        GoogleAuthProvider,
        OAuthProvider,
        PhoneAuthProvider;
import 'package:flutter/foundation.dart';
import 'package:flutter_facebook_login/flutter_facebook_login.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_login/src/models/user.dart';
import 'package:ocg_login/src/repository/login/base.dart';
import 'package:ocg_login/src/repository/user.dart';
import 'package:sign_in_with_apple/sign_in_with_apple.dart';

class LoginFirebaseRepository extends LoginRepository {
  final FirebaseAuth firebaseAuth;
  final TokenRepository tokenRepository;
  final LoginUserRepository userRepository;

  LoginFirebaseRepository(this.tokenRepository, {this.userRepository})
      : this.firebaseAuth = FirebaseAuth.instance;

  @override
  Future<AppResponse<AppToken>> createUserWithEmailAndPassword(
      String email, String password) async {
    try {
      var userCredential = await firebaseAuth.createUserWithEmailAndPassword(
          email: email, password: password);
      var token = AppToken.named(userId: userCredential.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials:
              LoginUser.emailCredentials(email: email, password: password),
          type: LoginType.Email);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<AppToken>> signInWithEmailAndPassword(
      String email, String password) async {
    try {
      var userCredential = await firebaseAuth.signInWithEmailAndPassword(
          email: email, password: password);
      var token = AppToken.named(userId: userCredential.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials:
              LoginUser.emailCredentials(email: email, password: password),
          type: LoginType.Email);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> isLoggedIn() async {
    var user = firebaseAuth.currentUser;
    return AppResponse.named(data: user != null);
  }

  @override
  Future<AppResponse<AppToken>> signInWithFacebook() async {
    try {
      FacebookLogin _facebookLogin = FacebookLogin();
      await _facebookLogin.logOut();
      var authentication = await _facebookLogin.logIn([]);
      switch (authentication.status) {
        case FacebookLoginStatus.cancelledByUser:
          return AppResponse.named(error: "Cancelled by user");
          break;
        case FacebookLoginStatus.loggedIn:
          AuthCredential credential = FacebookAuthProvider.getCredential(
            accessToken: authentication.accessToken.token,
          );
          var facebookResponse = await firebaseAuth.signInWithCredential(
            credential,
          );

          var token = AppToken.named(userId: facebookResponse.user.uid);
          await tokenRepository.saveToken(token);
          LoginUser user = LoginUser.named(
              uid: token.userId,
              credentials: LoginUser.facebookCredentials(
                facebookId: authentication.accessToken.userId,
                // TODO
              ),
              type: LoginType.Facebook);
          if (userRepository != null) {
            await userRepository.signUp(user);
          }
          return AppResponse.named(data: token);
        case FacebookLoginStatus.error:
          return AppResponse.named(error: authentication.errorMessage);
          break;
        default:
      }
      return AppResponse.named(error: "Facebook login error");
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "Facebook login error");
    }
  }

  @override
  Future<AppResponse<AppToken>> signWithGoogle() async {
    try {
      GoogleSignIn _googleSign = GoogleSignIn();
      await _googleSign.signOut();
      var response = await _googleSign.signIn();
      var authentication = await response.authentication;
      var credential = GoogleAuthProvider.getCredential(
          accessToken: authentication.accessToken,
          idToken: authentication.idToken);
      var googleResponse = await firebaseAuth.signInWithCredential(credential);
      var token = AppToken.named(userId: googleResponse.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials: LoginUser.googleCredentials(
              googleId: response.id,
              email: response.email,
              name: response.displayName,
              photoUrl: response.photoUrl),
          type: LoginType.Google);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "Google Login Error");
    }
  }

  String _verificationId;

  @override
  Future<AppResponse<String>> sendOTP(
      String countryCode, String mobileNumber) async {
    try {
      var error;
      String realMobile = "+" + countryCode + mobileNumber;
      _verificationId = null;
      await firebaseAuth.verifyPhoneNumber(
          phoneNumber: realMobile,
          verificationCompleted: (credential) {},
          verificationFailed: (e) {
            error = e;
            debugPrint("Error $e");
          },
          codeAutoRetrievalTimeout: (val) {},
          codeSent: (verificationId, [val]) {
            _verificationId = verificationId;
          },
          timeout: Duration(seconds: 6));

      if (error != null) {
        return AppResponse.named(error: 'Error $error');
      }
      for (var i = 0; i < 10; i++) {
        await Future.delayed(Duration(seconds: 1));
        if (_verificationId != null) {
          return AppResponse.named(data: realMobile);
        }
      }
      return AppResponse.named(error: "Timeout Exception");
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<AppToken>> verifyOTP(
      String mobileNumber, String otp) async {
    try {
      var credential = await firebaseAuth.signInWithCredential(
          PhoneAuthProvider.getCredential(
              verificationId: _verificationId, smsCode: otp));
      var token = AppToken.named(userId: credential.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials: LoginUser.mobileCredentials(mobile: mobileNumber),
          type: LoginType.Mobile);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<AppToken>> signInWithApple() async {
    try {
      if (!Platform.isIOS) {
        return AppResponse.named(error: "Only supported for ios");
      }
      var result = await SignInWithApple.getAppleIDCredential(scopes: [
        AppleIDAuthorizationScopes.email,
        AppleIDAuthorizationScopes.fullName,
      ]);
      var appleCredential = result.authorizationCode;
      final oAuthProvider = OAuthProvider(providerId: "apple.com");
      final credential = oAuthProvider.getCredential(
          accessToken: appleCredential, idToken: result.identityToken);
      var appleResponse = await firebaseAuth.signInWithCredential(credential);
      var token = AppToken.named(userId: appleResponse.user.uid);
      await tokenRepository.saveToken(token);
      LoginUser user = LoginUser.named(
          uid: token.userId,
          credentials: LoginUser.appleCredential(
            appleId: result.userIdentifier,
            email: result.email,
            name: result.givenName,
          ),
          type: LoginType.Google);
      if (userRepository != null) {
        await userRepository.signUp(user);
      }
      return AppResponse.named(data: token);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> signOut() async {
    try {
      GoogleSignIn().signOut();
      FacebookLogin().logOut();
      await firebaseAuth.signOut();
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<AppToken>> getToken() async {
    return await tokenRepository.getToken();
  }

  @override
  Future<AppResponse<bool>> sendResetPasswordEmail(String email) async {
    try {
      await firebaseAuth.sendPasswordResetEmail(email: email);
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }
}
